@@ -44,6 +44,7 @@ Client::Client(Window window, bool visible_already, ClientManager& cm)
     , keyMask_(this,  "keymask", RegexStr::fromStr(""))
     , keysInactive_(this,  "keys_inactive", RegexStr::fromStr(""))
     , pid_(this,  "pid", -1)
+    , pgid_(this, "pgid", -1)
     , pseudotile_(this,  "pseudotile", false)
     , ewmhrequests_(this, "ewmhrequests", true)
     , ewmhnotify_(this, "ewmhnotify", true)
@@ -55,6 +56,7 @@ Client::Client(Window window, bool visible_already, ClientManager& cm)
     , theme(*cm.theme)
     , settings(*cm.settings)
     , ewmh(*cm.ewmh)
+    , mostRecentThemeType(Theme::Type::Tiling)
 {
     stringstream tmp;
     window_id_str = WindowID(window).str();
@@ -80,7 +82,10 @@ Client::Client(Window window, bool visible_already, ClientManager& cm)
                 Root::get()->keys()->ensureKeyMask();
             }
             });
-    fullscreen_.changed().connect(this, &Client::updateEwmhState);
+    fullscreen_.changed().connect([this] {
+        updateEwmhState();
+        hook_emit({"fullscreen", fullscreen_() ? "on" : "off", WindowID(window_).str()});
+    });
 
     init_from_X();
 }
@@ -93,6 +98,7 @@ void Client::init_from_X() {
     last_size_ = float_size_;
 
     pid_ = Root::get()->X.windowPid(window_);
+    pgid_ = Root::get()->X.windowPgid(window_);
 
     update_title();
     update_wm_hints();
@@ -215,15 +221,7 @@ void Client::window_focus() {
 }
 
 const DecTriple& Client::getDecTriple() {
-    auto triple_idx = Theme::Type::Tiling;
-    if (fullscreen_()) {
-        triple_idx = Theme::Type::Fullscreen;
-    } else if (is_client_floated()) {
-        triple_idx = Theme::Type::Floating;
-    } else {
-        triple_idx = Theme::Type::Tiling;
-    }
-    return theme[triple_idx];
+    return theme[mostRecentThemeType];
 }
 
 void Client::setup_border(bool focused) {
@@ -232,6 +230,7 @@ void Client::setup_border(bool focused) {
 
 void Client::resize_fullscreen(Rectangle monitor_rect, bool isFocused) {
     dec->resize_outline(monitor_rect, theme[Theme::Type::Fullscreen](isFocused,urgent_()));
+    mostRecentThemeType = Theme::Type::Fullscreen;
 }
 
 void Client::raise() {
@@ -248,6 +247,7 @@ void Client::resize_tiling(Rectangle rect, bool isFocused, bool minimalDecoratio
     // only apply minimal decoration if the window is not pseudotiled
     auto themetype = (minimalDecoration && !pseudotile_())
             ? Theme::Type::Minimal : Theme::Type::Tiling;
+    mostRecentThemeType = themetype;
     auto& scheme = theme[themetype](isFocused, urgent_());
     if (this->pseudotile_) {
         auto inner = this->float_size_;
@@ -415,6 +415,7 @@ void Client::resize_floating(Monitor* m, bool isFocused) {
               m->rect.y + m->pad_up() - rect.height + space,
               m->rect.y + m->rect.height - m->pad_up() - m->pad_down() - space);
     dec->resize_inner(rect, theme[Theme::Type::Floating](isFocused,urgent_()));
+    mostRecentThemeType = Theme::Type::Floating;
 }
 
 Rectangle Client::outer_floating_rect() {
@@ -479,6 +480,11 @@ void Client::set_urgent(bool state) {
         // nothing to do
         return;
     }
+    if (this == manager.focus() && state == true) {
+        // ignore it if the focused client wants to be urgent
+        // because it will be removed by window_focus() anyway
+        return;
+    }
     set_urgent_force(state);
 }
 
@@ -504,6 +510,7 @@ void Client::set_urgent_force(bool state) {
 
     XSetWMHints(g_display, this->window_, wmh);
     XFree(wmh);
+    ewmh.updateWindowState(this);
     // report changes to tags
     tag_set_flags_dirty();
 }
@@ -622,6 +629,7 @@ void Client::fuzzy_fix_initial_position() {
     int extreme_x = float_size_.x;
     int extreme_y = float_size_.y;
     const auto& t = theme[Theme::Type::Floating];
+    mostRecentThemeType = Theme::Type::Floating;
     auto r = t.active.inner_rect_to_outline(float_size_);
     extreme_x = std::min(extreme_x, r.x);
     extreme_y = std::min(extreme_y, r.y);
