@@ -7,6 +7,7 @@
 #include "attribute_.h"
 #include "object.h"
 #include "regexstr.h"
+#include "theme.h"
 #include "types.h"
 #include "x11-types.h"
 
@@ -17,7 +18,6 @@ class Slice;
 class HSTag;
 class Monitor;
 class Settings;
-class Theme;
 class ClientManager;
 
 class Client : public Object {
@@ -63,6 +63,7 @@ public:
     Attribute_<RegexStr> keyMask_; // regex for key bindings that are active on this window
     Attribute_<RegexStr> keysInactive_; // regex for key bindings that are inactive on this window
     Attribute_<int>  pid_;
+    Attribute_<int>  pgid_;
     Attribute_<bool> pseudotile_; // only move client but don't resize (if possible)
     Attribute_<bool> ewmhrequests_; // accept ewmh-requests for this client
     Attribute_<bool> ewmhnotify_; // send ewmh-notifications for this client
@@ -131,6 +132,7 @@ private:
     Ewmh& ewmh;
     std::string tagName();
     const DecTriple& getDecTriple();
+    Theme::Type mostRecentThemeType;
 };
 
 
