@@ -7,9 +7,6 @@
 class Client;
 class HSTag;
 
-void floating_init();
-void floating_destroy();
-
 // utilities
 int char_to_direction(char ch);
 int find_rectangle_in_direction(RectangleIdxVec& rects, int idx, Direction dir);
@@ -22,5 +19,6 @@ bool floating_focus_direction(Direction dir);
 bool floating_shift_direction(Direction dir);
 bool floating_resize_direction(HSTag* tag, Client* client, Direction dir);
 
+Point2D floatingSmartPlacement(HSTag* tag, Client* client, Point2D area, int gap);
 
 #endif
