@@ -5,6 +5,9 @@
 #include <memory>
 #include <string>
 
+#include "fixprecdec.h"
+#include "link.h"
+#include "object.h"
 #include "types.h"
 
 class Client;
@@ -19,7 +22,7 @@ class TreeInterface;
 /*! A class representing an entire tree of frames that provides
  * the tiling commands and other common actions on the frame tree
  */
-class FrameTree : public std::enable_shared_from_this<FrameTree> {
+class FrameTree : public Object {
 public:
     FrameTree(HSTag* tag, Settings* settings);
     void foreachClient(std::function<void(Client*)> action);
@@ -49,7 +52,7 @@ public:
         End,
     };
 
-    bool resizeFrame(double delta, Direction dir);
+    bool resizeFrame(FixPrecDec delta, Direction dir);
 
     // Commands
     int cycleSelectionCommand(Input input, Output output);
@@ -68,16 +71,16 @@ public:
     void setLayoutCompletion(Completion& complete);
 public: // soon to be come private:
     std::shared_ptr<Frame> root_;
+    Link_<Frame> rootLink_;
+    //! replace a node in the frame tree, either modifying old's parent or the root_
+    void replaceNode(std::shared_ptr<Frame> old, std::shared_ptr<Frame> replacement);
 private:
     static std::shared_ptr<FrameLeaf> findEmptyFrameNearFocusGeometrically(std::shared_ptr<Frame> subtree);
     //! cycle the frames within the current tree
     void cycle_frame(std::function<size_t(size_t,size_t)> indexAndLenToIndex);
     void cycle_frame(int delta);
-    //! try to resemble a given raw frame tree given by the FrameParser
     void applyFrameTree(std::shared_ptr<Frame> target,
                         std::shared_ptr<RawFrameNode> source);
-    //! replace a node in the frame tree, either modifying old's parent or the root_
-    void replaceNode(std::shared_ptr<Frame> old, std::shared_ptr<Frame> replacement);
     static std::shared_ptr<TreeInterface> treeInterface(
         std::shared_ptr<Frame> frame,
         std::shared_ptr<FrameLeaf> focus);
